
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <errno.h>

#define HIGH 0x01
#define LOW  0x0

#define TRUE  0x01
#define FALSE 0x0

/*
 *      EXAR ioctls
 */
//#define 	FIOQSIZE		0x5460 
#define		EXAR_READ_REG      	(FIOQSIZE + 1)
#define 	EXAR_WRITE_REG     	(FIOQSIZE + 2)

struct xrioctl_rw_reg {
	unsigned char reg;
	unsigned char regvalue;
};
 
int xrfd = -1;

int read_reg(int reg, int *value)
{
  struct xrioctl_rw_reg regread;
  int result;

  regread.reg = reg;
  result = ioctl(xrfd, EXAR_READ_REG, &regread);
  if (result < 0)
    printf("Cannot do EXAR_READ_REG ioctl (%d)\n", result);
  else
    (*value) = regread.regvalue;
 
  return result;
}

#define EEPROM_REGISTER 0x8e
#define XREEPROMTIME0	0xEFFF

#define BYTE unsigned char
#define BOOL unsigned char
#define DWORD unsigned int

BYTE byOldState = 0;

unsigned char Read()
{
	struct xrioctl_rw_reg regread;
	int result;

	regread.reg = EEPROM_REGISTER;
	result = ioctl(xrfd, EXAR_READ_REG, &regread);
	if (result < 0)
	{
		printf("Cannot do EXAR_READ_REG ioctl (%d)\n", result);
		return result;
	}
	else
	{
		return regread.regvalue;
	}
}

void Write(unsigned char ucData)
{
	struct xrioctl_rw_reg regwrite;
	int result;

	regwrite.reg = EEPROM_REGISTER;
	regwrite.regvalue = ucData;
	result = ioctl(xrfd, EXAR_WRITE_REG, &regwrite);
	if (result < 0)
	{
		printf("Cannot do EXAR_READ_REG ioctl (%d)\n", result);
	}
}



void WriteClk( BOOL bValue )
{
	BYTE byVal;
	byVal = byOldState;
	if(bValue)
	{
		byVal |= 0x10;
		Write(byVal);
		byOldState |= 0x10;
	}
	else
	{
		byVal &= ~0x10;
		Write(byVal);
		byOldState &= ~0x10;
	}
}

void WriteCs( BOOL bValue )
{
	BYTE byVal;
	byVal = byOldState;
	if(bValue)
	{
		byVal |= 0x20;
		Write(byVal);
		byOldState |= 0x20;
	}
	else
	{
		byVal &= ~0x20;
		Write(byVal);
		byOldState &= ~0x20;
	}
}

void WriteDi( BOOL bValue )
{
	BYTE byVal;
	byVal = byOldState;
	if(bValue)
	{
		byVal |= 0x40;
		Write(byVal);
		byOldState |= 0x40;
	}
	else
	{
		byVal &= ~0x40;
		Write(byVal);
		byOldState &= ~0x40;
	}
}

unsigned char ReadDo()
{
	BYTE byVal;
	byVal = Read();
	byVal &= 0x80;
	return byVal;
}

void XRSleep(DWORD dwCount)
{	
	usleep (1000);
}

DWORD ReOrder(DWORD dwData, BYTE byPos)
{
	BYTE i;

	DWORD dwTemp = 0;
	DWORD dwTemp2 = 0;
	DWORD byBit = 0;
	
	if(byPos == 6)
		byBit = 0x20;
	else if(byPos == 16)
		byBit = 0x8000;
	else
		byBit = 0;
	
	for(i=0; i<byPos; i++)
	{
		dwTemp = dwData & byBit;
		dwData = dwData << 1;
		dwTemp = dwTemp >> (byPos-1-i);
		dwTemp2 |= dwTemp;
	}
	dwData = dwTemp2;
  	return dwData;
}

DWORD ReadE2prom( DWORD dwAddrInRom )
{
	DWORD i, j;
	DWORD dwTemp = 99;
	BYTE byTemp = 0;
	int  dwRead = 0;

	dwAddrInRom = ReOrder(dwAddrInRom, 0x06);

	WriteCs(HIGH);
	XRSleep(XREEPROMTIME0);
	WriteDi(HIGH);  //DI = 1
	XRSleep(XREEPROMTIME0);

	WriteClk(HIGH);
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);
	WriteClk(HIGH);  //DI = 1
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);
	WriteDi(LOW);
	XRSleep(XREEPROMTIME0);
	
	WriteClk(HIGH);  //DI = 0
	XRSleep(XREEPROMTIME0);

	for(i=0; i<6; i++)
	{
		dwTemp = dwAddrInRom & 0x01;
		WriteClk(LOW);
		WriteDi((BOOL) dwTemp);
		dwAddrInRom = dwAddrInRom >> 1;

		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);
		XRSleep(XREEPROMTIME0);
	}

	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);
	WriteClk(HIGH);
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);


	//Read D0 out
	for(j=0; j<16; j++)
	{
		WriteClk(HIGH);
		byTemp = ReadDo();
		if(byTemp)
		{
			dwRead |= 0x01;
			dwRead = dwRead << 1;
		}
		else
		{
			dwRead &= 0xfffe;
			dwRead = dwRead << 1;
		}
		XRSleep(XREEPROMTIME0);
		WriteClk(LOW);
		XRSleep(XREEPROMTIME0);
	}
	WriteCs(LOW);
	XRSleep(XREEPROMTIME0);
	dwRead = dwRead >> 1;

	return (DWORD) dwRead;
}

void WriteE2prom( DWORD dwAddrInRom, DWORD dwData )
{
	DWORD i;
	DWORD dwTemp = 99;
	BYTE byTemp = 0;

	dwAddrInRom = ReOrder(dwAddrInRom, 0x06);

	WriteCs(HIGH);
	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);
	WriteDi(HIGH);  //DI = 1
	XRSleep(XREEPROMTIME0);

	WriteClk(HIGH);
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);

	WriteDi(LOW);
	XRSleep(XREEPROMTIME0);
	WriteClk(HIGH);  //DI = 0
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);
	WriteDi(HIGH);
	XRSleep(XREEPROMTIME0);
	
	WriteClk(HIGH);  //DI = 1
	XRSleep(XREEPROMTIME0);

	for(i=0; i<6; i++)
	{
		dwTemp = dwAddrInRom & 0x01;
		WriteClk(LOW);
		WriteDi((BOOL) dwTemp);
		dwAddrInRom = dwAddrInRom >> 1;

		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);
		XRSleep(XREEPROMTIME0);
	}

	dwData = ReOrder(dwData, 16);
	for(i=0; i<16; i++)
	{
		dwTemp = dwData & 0x01;
		WriteClk(LOW);
		WriteDi((BOOL) dwTemp);
		dwData = dwData >> 1;

		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);
		XRSleep(XREEPROMTIME0);
	}
	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);
	WriteCs(LOW);
	XRSleep(XREEPROMTIME0);
}

void WriteEnable( BOOL bTrue )
{
	DWORD i;

	WriteCs(HIGH);
	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);
	WriteDi(HIGH);  //DI = 1
	XRSleep(XREEPROMTIME0);

	WriteClk(HIGH);
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);

	WriteDi(LOW);
	XRSleep(XREEPROMTIME0);
	WriteClk(HIGH);  //DI = 0
	XRSleep(XREEPROMTIME0);
	WriteClk(LOW);
	WriteDi(LOW);
	XRSleep(XREEPROMTIME0);
	
	WriteClk(HIGH);  //DI = 0
	XRSleep(XREEPROMTIME0);

	if(bTrue)
	{
		WriteClk(LOW);
		WriteDi(HIGH);
		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);  //DI = 1
		XRSleep(XREEPROMTIME0);
		
		WriteClk(LOW);
		WriteDi(HIGH);
		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);  //DI = 1
		XRSleep(XREEPROMTIME0);
	}
	else
	{
		WriteClk(LOW);
		WriteDi(LOW);
		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);  //DI = 1
		XRSleep(XREEPROMTIME0);
		
		WriteClk(LOW);
		WriteDi(LOW);
		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH);  //DI = 1
		XRSleep(XREEPROMTIME0);
	}

	for(i=0; i<5; i++)
	{
		WriteClk(LOW);
		XRSleep(XREEPROMTIME0);
		WriteClk(HIGH); 
		XRSleep(XREEPROMTIME0);
	}

	WriteClk(LOW);
	XRSleep(XREEPROMTIME0);
	WriteCs(LOW);
	XRSleep(XREEPROMTIME0);
}

int main(int argc, char * argv[]) 
{

	FILE *fp;
	int value, i, j;
	unsigned short x;

	char * device = "/dev/ttyXR0";
	printf("try to open %s\n", device);

	xrfd = open(device, O_RDWR | O_NOCTTY | O_NDELAY);

	if(xrfd == -1) 
	{
		printf("Can't open device %s\n", device);
		return 1;
	}
	else
		printf("%s opened\n", device);

	if(read_reg(0x8D, &value) >= 0)
		printf("Internal Device ID : 0x%x\n", value);
	else
	{
		printf("Error: read_reg failed. Can not continue. Please check.\n");
		return 1;
	}

      	/* open the file */
      	fp = fopen("eeprom.txt", "r");
      	if (fp == NULL) 
	{
		printf("ERROR: Couldn't open eeprom.txt for reading.\n");
	}
	else
	{
#if 1
		printf("EEPROM before programming:\n");
		printf("0x0 = 0x%x, 0x1 = 0x%x\n", ReadE2prom(0x00), ReadE2prom(0x01));
		printf("0x2 = 0x%x, 0x3 = 0x%x\n", ReadE2prom(0x02), ReadE2prom(0x03));
#endif
		j = 0;
		printf("EEPROM programming begins...\n");
		WriteEnable(TRUE);
		while (fscanf(fp, "%d\n", &x) == 1)
		{		
			WriteE2prom(j, x);
			printf("%d = 0x%x\n", j++, x);
		}
		WriteEnable(FALSE);
		printf("EEPROM programming ends.\n");

#if 1
		printf("EEPROM after programming:\n");
		printf("0x0 = 0x%x, 0x1 = 0x%x\n", ReadE2prom(0x00), ReadE2prom(0x01));
		printf("0x2 = 0x%x, 0x3 = 0x%x\n", ReadE2prom(0x02), ReadE2prom(0x03));
#endif
		
		/* close the file */
		fclose(fp);
	}

	close(xrfd);

	return 1;
}


